# frozen_string_literal: true

require 'omniauth'
require 'oauth2'
require 'omniauth-oauth2'

module OmniAuth
  module Strategies
    class AinewsAuth < OmniAuth::Strategies::OAuth2
      option :name, :ainews_auth

      option :client_options, {
        site: 'https://dev-auth.dev.b2b.aitudala.net',
        token_url: '/login/v1/access-token',
        authorize_url: '/login/authorize',
        token_method: :post,
        auth_scheme: :request_body,
        extract_access_token: proc do |client, hash| # proc to extract the token
          hash = hash['data']['attributes']
          token = hash.delete('access_token') || hash.delete(:access_token)
          token && ::OAuth2::AccessToken.new(client, token, hash)
        end
      }

      option :token_params, {
        grant_type: 'authorization_code'
      }

      def build_access_token
        options.token_params.merge!({ auth_code: request.params['auth_code'], state: request.params['state'] })
        super
      end

      option :pkce, true

      uid{ raw_info.dig('data', 'id') }

      info do
        {
          :id => raw_info.dig('data', 'id'),
          :name => raw_info.dig('data', 'attributes', 'name'),
          :email => raw_info.dig('data', 'attributes', 'email'),
          :phone => raw_info.dig('data', 'attributes', 'phone'),
        }
      end

      extra do
        {
          'raw_info' => raw_info
        }
      end

      def raw_info
        @raw_info ||= access_token.post(
          '/login/v1/get-user', body: options[:token_params].merge(
            { access_token: access_token.token }
          )
        ).parsed
      end
    end
  end
end
